# makeFragmentTex.py
#
# The code in this file is part of PyXPlot
# <http://www.pyxplot.org.uk>
#
# Copyright (C) 2006-2012 Dominic Ford <coders@pyxplot.org.uk>
#
# $Id: makeFragmentTex.py 1267 2012-07-15 00:11:04Z dcf21 $
#
# PyXPlot is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# You should have received a copy of the GNU General Public License along with
# PyXPlot; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

# ----------------------------------------------------------------------------

# Turn scripts of examples into latex

import os,sys,glob,re,subprocess

os.system("rm -Rf fragments/tex")
os.system("mkdir  fragments/tex")

if len(sys.argv)>=2: pyxplot = sys.argv[1]
else               : pyxplot = "../bin/pyxplot"

def line_texify(line,userInput):
  line = re.sub(r'[\\]', r'gpzywxqqq', line) # LaTeX does not like backslashs
  line = re.sub(r'[_]', r'\\_', line) # LaTeX does not like underscores....
  line = re.sub(r'[&]', r'\\&', line) # LaTeX does not like ampersands....
  line = re.sub(r'[%]', r'\\%', line) # LaTeX does not like percents....
  line = re.sub(r'[$]', r'\\$', line) # LaTeX does not like $s....
  line = re.sub(r'[{]', r'\\{', line) # LaTeX does not like {s....
  line = re.sub(r'[}]', r'\\}', line) # LaTeX does not like }s....
  line = re.sub(r'[\^]', r'\\^{}', line) # LaTeX does not like carets....
  line = re.sub(r'[~]', r'$\\sim$', line) # LaTeX does not like tildas....
  line = re.sub(r'[<]', r'$<$', line) # LaTeX does not like < outside of mathmode....
  line = re.sub(r'[>]', r'$>$', line) # LaTeX does not like > outside of mathmode....
  line = re.sub(r'gpzywxqqq', r'$\\backslash$', line) # LaTeX does not like backslashs
  line = re.sub(r' ', r'~', line)
  if not userInput:
    line = re.sub(r'[#]', r'\\#', line) # LaTeX does not like #s....
  else:
    hashPos = line.find("#")
    if hashPos<0: return line
    line = r"%s}}{{\it \#%s"%(line[:hashPos] , line[hashPos+1:])
  return line

files = glob.glob("fragments/*.ppl")
files.sort()
for fname in files:
  print "Converting fragment to latex <%s>..."%os.path.split(fname)[1]
  out       = os.path.join("fragments","tex",os.path.split(fname)[1][:-4]+".tex")
  linecount = 0
  lines     = open(fname).readlines()
  out       = open(out,"w")
  first     = True
  prompt    = "pyxplot"
  for i in range(len(lines)):
    if (len(lines[i].strip())<1): continue
    if (not first): out.write("\\newline\n")
    first = False
    out.write(r"\noindent\texttt{%s> \textbf{%s}}"%(prompt,line_texify(lines[i].rstrip(),1)))
    if lines[i].strip()[-1]=="\\":
      prompt = "......."
      continue
    prompt = "pyxplot"
    sp = subprocess.Popen([pyxplot], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    o  = sp.communicate(input="\n".join([ ll.strip() for ll in lines[0:i+1]])) # returns (stdout,stderr)
    if (len(o[1])>0): raise RuntimeError("pyxplot failed: %s"%o[1])
    olines = o[0].strip().split('\n')
    if (olines==['']): olines=[]
    linecountNew = len(olines)
    if linecountNew<=linecount: continue
    olines = olines[linecount:]
    linecount = linecountNew
    for line in olines:
      if (len(line.strip())<1): continue
      out.write("\\newline\n\\noindent\\texttt{%s}"%line_texify(line,0))
  out.write("\n")
  out.close()



# parser_data.py
#
# The code in this file is part of Pyxplot
# <http://www.pyxplot.org.uk>
#
# Copyright (C) 2006-2012 Dominic Ford <coders@pyxplot.org.uk>
#               2008-2012 Ross Church
#
# $Id: parser_data.py 1261 2012-07-11 21:38:05Z dcf21 $
#
# Pyxplot is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# You should have received a copy of the GNU General Public License along with
# Pyxplot; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

# ----------------------------------------------------------------------------

# Generate table of positions of variables by name
globalVarNames = []
globalVarTable = {}

def processVarTable(vt, directive, setOption):
 global globalVarNames,globalVarTable
 if '%' in directive: return
 key = "%s_%s"%(directive,setOption)
 globalVarTable[key] = vt
 for name,pos in vt.iteritems():
  if name not in globalVarNames: globalVarNames.append(name)

def printVarTable():
 global globalVarNames,globalVarTable,f_h,f_c
 globalVarNames.sort()
 for i in range(len(globalVarNames)):
   f_h.write("#define PARSE_INDEX_%s %d\n"%(sanitize(globalVarNames[i]),i))
 keys = globalVarTable.keys()
 keys.sort()
 for k in keys:
   d = globalVarTable[k]
   f_h.write("extern const int PARSE_TABLE_%s[];\n"%k)
   f_c.write("const int PARSE_TABLE_%s[] = {"%k)
   for i in range(len(globalVarNames)):
     item = globalVarNames[i]
     if (i!=0)   : f_c.write(",")
     if item in d: f_c.write("%d"%d[item])
     else        : f_c.write("-1");
   f_c.write("};\n")

# Generate specification of commands that Pyxplot recognises

def sanitize(instr):
  outstr = ""
  for i in range(len(instr)):
    if (instr[i].isalnum() or instr[i]=='_'): outstr+=instr[i]
  return outstr

f_in = open("buildScripts/parser_data.dat","r")

# Output C files containing command definitions

f_h  = open("src/parser/cmdList.h","w")
f_c  = open("src/parser/cmdList.c","w")
includeKeys = {}

f_h.write("""// This file autogenerated by parser_data.py

#ifndef _CMDLIST_H
#define _CMDLIST_H 1

extern const char ppl_cmdList[];
""")

f_c.write("""// This file autogenerated by parser_data.py

const char ppl_cmdList[] = "\\
""")

# Loop through command definitions

linecount = 0
for line in f_in:
  linecount+=1
  line = line.strip();
  if len(line)<1: continue
  if line.startswith("#"): continue

  outline        = ""
  directive      = "undefined"
  setoption      = ""
  stack          = []
  stack_varnames = []
  listsizes      = []
  varnames       = {"directive":0 , "editno":1, "set_option":2, "X":3}
  vartable       = varnames.copy()
  varcount       = len(varnames)
  words = line.split()
  for word in words:
    if word in ["=","(","~",")","{","}","<","|",">","["]: # Grammar characters are passed straight through
      outline += "%s "%word
      if word=="[": # Lists are placed on the stack, and the number of variables in each list counted
        stack.append(varcount)
        stack_varnames.append(varnames)
        varcount=1 # Start counting from variable 1 inside a list, as first value is pointer to next list item
        varnames={}
      continue
    parts = word[1:].split(":")
    parts[0] = word[0]+parts[0] # If first character is a colon, it should match a literal :, not act as a separator
    subparts = parts[0].split("@") # Match string has form plot@1, meaning 'p' is short for 'plot'
    if len(parts)<2:
      parts.append("X") # If output variable name is not specified, stick it in 'X'.
    if len(subparts)>1:
      if subparts[1]=="n": subparts[1]=-1 # An auto-complete length of 'n' is stored as -1
      parts.append("%s"%subparts[1])
      parts[0] = subparts[0]
    else:
      parts.append("%s"%len(parts[0])) # If no auto-complete length is specified, whole string must be matched
    if len(parts)==3: parts.insert(2,"") # At this point, parts is [ match string , output variable , string to store (blank if equals match string) , auto-complete len ]
    assert len(parts)==4, "Syntax error in word '%s'."%word
    varname = parts[1]
    if word.startswith("]:"):
      parts = [ "]" , word[2:] , "" , "1" ]
      listsizes.append(varcount)
      varcount=stack.pop()
      for key,item in varnames.iteritems(): stack_varnames[-1]["%s_%s"%(key,varname)] = item
      varnames=stack_varnames.pop()
    if varname=='directive': # Directive names are stored for use in #defines to convert variable names into output slot numbers
      if parts[2]=="": directive = parts[0]
      else           : directive = parts[2]
    if varname=='set_option': # Set options are also used in #defines
      if parts[2]=="": setoption = parts[0]+"_"
      else           : setoption = parts[2]+"_"
      if (setoption[0]=="%"): setoption=""
    if varname not in varnames:
      varnames[varname] = varcount
      vartable[varname] = varcount
      if   (parts[0]=="%p"): varcount += 2 # Position vectors require 2 or 3 slots
      elif (parts[0]=="%P"): varcount += 3
      else                 : varcount += 1 # Varcount keeps track of the slot number to place the next variable in
    elif (parts[0] in ["%p","%P"]): print "Danger in command %s: sharing position variable name with other variables of different lengths"%directive
    outnum = varnames[varname]
    parts.append("%s"%outnum) # parts[4] = slot number
    if word.startswith("]:"):
      initial = ""
      parts.append("%s"%listsizes[-1]) # parts[5] = list length for ] (number of slots needed for this list's variables)
    else:
      initial = "@"
      parts.append("0") # ... or zero otherwise
    outline += initial + "@".join(parts) + " "
  f_c.write("%d "%(varcount)) # First word on each statement definition line is the number of variables in the root slotspace
  f_c.write("%s\\n\\\n"%outline)
  processVarTable(vartable,directive,setoption)
  for i,j in varnames.iteritems():
   if '%' not in directive:
    key = "PARSE_%s_%s%s"%(directive,setoption,sanitize(i))
    if (key in includeKeys) and (includeKeys[key]!=j): print "Repetition of key %s"%key
    includeKeys[key] = j
    f_h.write("#define %s %d\n"%(key,j)) # Write #defines to convert variable names into slot numbers

# Finish up

f_c.write("""";\n""");
printVarTable()
f_h.write("""\n\n#endif"""); f_h.close()
f_c.close()




# manpage_pyxplot.py
#
# The code in this file is part of Pyxplot
# <http://www.pyxplot.org.uk>
#
# Copyright (C) 2006-2012 Dominic Ford <coders@pyxplot.org.uk>
#               2008-2012 Ross Church
#
# $Id: manpage_pyxplot.py 1273 2012-07-20 00:18:37Z dcf21 $
#
# Pyxplot is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# You should have received a copy of the GNU General Public License along with
# Pyxplot; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

# ----------------------------------------------------------------------------

# Generate manpage for Pyxplot

import sys

docpath     = sys.argv[1]
author      = open("AUTHORS","r").read()
description = ""

f = open("README","r")
state = 0
for line in f.readlines():
 if   (line[0:2] == "1."): state = 1
 elif (line[0:2] == "2."): state = 2
 elif (state == 1)       : description += line

sys.stdout.write(r"""
.\" pyxplot.1
.\"
.\" The manpage in this file is part of Pyxplot
.\" <http://www.pyxplot.org.uk>
.\"
.\" Copyright (C) 2006-2012 Dominic Ford <coders@pyxplot.org.uk>
.\"               2008-2012 Ross Church
.\"
.\" $Id: manpage_pyxplot.py 1273 2012-07-20 00:18:37Z dcf21 $
.\"
.\" Pyxplot is free software; you can redistribute it and/or modify it under the
.\" terms of the GNU General Public License as published by the Free Software
.\" Foundation; either version 2 of the License, or (at your option) any later
.\" version.
.\"
.\" You should have received a copy of the GNU General Public License along with
.\" Pyxplot; if not, write to the Free Software Foundation, Inc., 51 Franklin
.\" Street, Fifth Floor, Boston, MA  02110-1301, USA
.\"
.\" ----------------------------------------------------------------------------

.\" Man page for pyxplot

.TH PYXPLOT 1
.SH NAME
pyxplot \- a multi-purpose command-line data processing, vector graphics and graph-plotting tool.
.SH SYNOPSIS
.B pyxplot
[file ...]
.SH DESCRIPTION
%s
Full documentation can be found in:
%s
.SH COMMAND LINE OPTIONS
  \-h, \-\-help:       Display this help.
  \-v, \-\-version:    Display version number.
  \-q, \-\-quiet:      Turn off initial welcome message.
  \-V, \-\-verbose:    Turn on initial welcome message.
  \-c, \-\-color:      Use colored highlighting of output.
  \-m, \-\-monochrome: Turn off coloured highlighting.
.SH AUTHORS
%s.
.SH BUGS AND USER FORUMS
To report bugs in Pyxplot, or to meet other Pyxplot users in our forums, please
visit us on sourceforge: <http://sourceforge.net/projects/pyxplot/>.
.SH CREDITS
Matthew Smith, Michael Rutter, Zoltan Voros and John Walker have all
contributed code to Pyxplot.  We welcome bug reports, which can be submitted to
our project page on Sourceforge, and thank the many testers who have already
made significant contributions to the project by helping us to track down bugs.
.SH "SEE ALSO"
.BR pyxplot_watch (1), gnuplot (1)
"""%(description,docpath,author))




# makeFigureEps.py
#
# The code in this file is part of PyXPlot
# <http://www.pyxplot.org.uk>
#
# Copyright (C) 2006-2012 Dominic Ford <coders@pyxplot.org.uk>
#
# $Id: makeFigureEps.py 1281 2012-07-29 18:33:42Z dcf21 $
#
# PyXPlot is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# You should have received a copy of the GNU General Public License along with
# PyXPlot; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

# ----------------------------------------------------------------------------

# Turn scripts of examples into latex

import glob,os,sys,re

if len(sys.argv)>=2: pyxplot = sys.argv[1]
else               : pyxplot = "../bin/pyxplot"

os.system("rm -Rf examples/eps")
os.system("mkdir  examples/eps")

files = glob.glob("examples/ex_*.ppl")
files.sort()
for fname in files:
  print "Working on example <%s>..."%os.path.split(fname)[1]
  status = os.system("%s %s"%(pyxplot,fname))
  if (status): raise RuntimeError("pyxplot failed")

# Make pdf and png versions of all figures
files = glob.glob("examples/eps/*.eps")
files.sort()
for eps in files:
  print "Converting example <%s> to pdf..."%os.path.split(eps)[1]
  pdf = re.sub(r"\.eps",".pdf",eps)
  #png = re.sub(r"\.eps",".png",eps)
  os.system("gs -dQUIET -dSAFER -P- -dBATCH -dNOPAUSE -dEPSCrop -sDEVICE=pdfwrite -sOutputFile=%s %s"%(pdf,eps))
  #os.system("gs -dQUIET -dSAFER -P- -dBATCH -dNOPAUSE -dEPSCrop -sDEVICE=png16m -r72 -dGraphicsAlphaBits=4 -dTextAlphaBits=4 -sOutputFile=%s %s"%(png,eps))




# colorlistGenerate.py
#
# The code in this file is part of Pyxplot
# <http://www.pyxplot.org.uk>
#
# Copyright (C) 2006-2012 Dominic Ford <coders@pyxplot.org.uk>
#               2008-2012 Ross Church
#
# $Id: colorlistGenerate.py 1261 2012-07-11 21:38:05Z dcf21 $
#
# Pyxplot is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# You should have received a copy of the GNU General Public License along with
# Pyxplot; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

# ----------------------------------------------------------------------------

color_list={
"greenYellow": [0.15, 0, 0.69, 0],
"yellow": [0, 0, 1, 0],
"goldenRod": [0, 0.10, 0.84, 0],
"dandelion": [0, 0.29, 0.84, 0],
"apricot": [0, 0.32, 0.52, 0],
"peach": [0, 0.50, 0.70, 0],
"melon": [0, 0.46, 0.50, 0],
"yellowOrange": [0, 0.42, 1, 0],
"orange": [0, 0.61, 0.87, 0],
"burntOrange": [0, 0.51, 1, 0],
"bitterSweet": [0, 0.75, 1, 0.24],
"redOrange": [0, 0.77, 0.87, 0],
"mahogany": [0, 0.85, 0.87, 0.35],
"maroon": [0, 0.87, 0.68, 0.32],
"brickRed": [0, 0.89, 0.94, 0.28],
"red": [0, 1, 1, 0],
"orangeRed": [0, 1, 0.50, 0],
"rubineRed": [0, 1, 0.13, 0],
"wildStrawberry": [0, 0.96, 0.39, 0],
"salmon": [0, 0.53, 0.38, 0],
"carnationPink": [0, 0.63, 0, 0],
"magenta": [0, 1, 0, 0],
"violetRed": [0, 0.81, 0, 0],
"rhodamine": [0, 0.82, 0, 0],
"mulberry": [0.34, 0.90, 0, 0.02],
"redViolet": [0.07, 0.90, 0, 0.34],
"fuchsia": [0.47, 0.91, 0, 0.08],
"lavender": [0, 0.48, 0, 0],
"thistle": [0.12, 0.59, 0, 0],
"orchid": [0.32, 0.64, 0, 0],
"darkOrchid": [0.40, 0.80, 0.20, 0],
"purple": [0.45, 0.86, 0, 0],
"plum": [0.50, 1, 0, 0],
"violet": [0.79, 0.88, 0, 0],
"royalPurple": [0.75, 0.90, 0, 0],
"blueViolet": [0.86, 0.91, 0, 0.04],
"periwinkle": [0.57, 0.55, 0, 0],
"cadetBlue": [0.62, 0.57, 0.23, 0],
"cornflowerBlue": [0.65, 0.13, 0, 0],
"midnightBlue": [0.98, 0.13, 0, 0.43],
"navyBlue": [0.94, 0.54, 0, 0],
"royalBlue": [1, 0.50, 0, 0],
"blue": [1, 1, 0, 0],
"cerulean": [0.94, 0.11, 0, 0],
"cyan": [1, 0, 0, 0],
"processBlue": [0.96, 0, 0, 0],
"skyBlue": [0.62, 0, 0.12, 0],
"turquoise": [0.85, 0, 0.20, 0],
"tealBlue": [0.86, 0, 0.34, 0.02],
"aquamarine": [0.82, 0, 0.30, 0],
"blueGreen": [0.85, 0, 0.33, 0],
"emerald": [1, 0, 0.50, 0],
"jungleGreen": [0.99, 0, 0.52, 0],
"seaGreen": [0.69, 0, 0.50, 0],
"green": [1, 0, 1, 0],
"forestGreen": [0.91, 0, 0.88, 0.12],
"pineGreen": [0.92, 0, 0.59, 0.25],
"limeGreen": [0.50, 0, 1, 0],
"yellowGreen": [0.44, 0, 0.74, 0],
"springGreen": [0.26, 0, 0.76, 0],
"oliveGreen": [0.64, 0, 0.95, 0.40],
"rawSienna": [0, 0.72, 1, 0.45],
"sepia": [0, 0.83, 1, 0.70],
"brown": [0, 0.81, 1, 0.60],
"tan": [0.14, 0.42, 0.56, 0],
"gray": [0, 0, 0, 0.50],
"grey": [0, 0, 0, 0.50],
"black": [0, 0, 0, 1],
"white": [0, 0, 0, 0],
"invisible": [-1, -1, -1, -1],
"null": [-1, -1, -1, -1],
"transparent": [-1, -1, -1, -1],
}

for greylevel in range(0,101):
 color_list["grey%02d"%greylevel] = [0, 0, 0, float(100-greylevel)/100]
 color_list["gray%02d"%greylevel] = [0, 0, 0, float(100-greylevel)/100]

color_names = color_list.keys()
color_names.sort()

# Write ppl_colors.h
output = open("src/settings/epsColors.h","w")
output.write("""// This file auto-generated by colorsGenerate.py

#ifndef _EPSCOLORS_H
#define _EPSCOLORS_H 1

""")

for i in range(len(color_names)): output.write("#define COLOR_%s %d\n"%(color_names[i].upper() , i+20000))
output.write("\n#ifndef _EPSCOLORS_C\n")
output.write("extern char  *SW_COLOR_STR[];\n")
output.write("extern int    SW_COLOR_DCL[];\n")
output.write("extern int    SW_COLOR_INT[];\n\n")
output.write("extern double SW_COLOR_CMYK_C[];\n")
output.write("extern double SW_COLOR_CMYK_M[];\n")
output.write("extern double SW_COLOR_CMYK_Y[];\n")
output.write("extern double SW_COLOR_CMYK_K[];\n")
output.write("#endif\n\n#endif\n")
output.close()

# Write ppl_colors.c
output = open("src/settings/epsColors.c","w")
output.write("""// This file auto-generated by colorsGenerate.py

#define _EPSCOLORS_C 1

#include "epsColors.h"

char  *SW_COLOR_STR[] = {""")
for i in range(len(color_names)): output.write(""" "%s" , """%color_names[i])
output.write("};\nint    SW_COLOR_DCL[] = {")
for i in range(len(color_names)): output.write(""" %d , """%len(color_names[i]))
output.write("};\nint    SW_COLOR_INT[] = {")
for i in range(len(color_names)): output.write(""" COLOR_%s , """%(color_names[i].upper()))
output.write("-1};\ndouble SW_COLOR_CMYK_C[] = {")
for i in range(len(color_names)): output.write(""" %f , """%(color_list[color_names[i]][0]))
output.write("};\ndouble SW_COLOR_CMYK_M[] = {")
for i in range(len(color_names)): output.write(""" %f , """%(color_list[color_names[i]][1]))
output.write("};\ndouble SW_COLOR_CMYK_Y[] = {")
for i in range(len(color_names)): output.write(""" %f , """%(color_list[color_names[i]][2]))
output.write("};\ndouble SW_COLOR_CMYK_K[] = {")
for i in range(len(color_names)): output.write(""" %f , """%(color_list[color_names[i]][3]))
output.write("};\n")

output.close()




# PYX_COLOURS.PY
#
# The code in this file is part of Pyxplot
# <http://www.pyxplot.org.uk>
#
# Copyright (C) 2006-2012 Dominic Ford <coders@pyxplot.org.uk>
#               2008-2012 Ross Church
#
# $Id: pyx_colors.py 1261 2012-07-11 21:38:05Z dcf21 $
#
# Pyxplot is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# You should have received a copy of the GNU General Public License along with
# Pyxplot; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

# ----------------------------------------------------------------------------

# This python produces figures showing all of the built-in colors defined in
# PyX

import pyx
from pyx import *
from math import *

# Fetch a list of all built-in colors defined in PyX
colors = [ i for i in pyx.color.cmyk.__dict__.keys() if isinstance(pyx.color.cmyk.__dict__[i],pyx.color.cmyk) ]
h_list = [ pyx.color.cmyk.__dict__[i].hsb().color['h']*2*pi for i in colors ]
s_list = [ pyx.color.cmyk.__dict__[i].hsb().color['s']      for i in colors ]

# Figure 1: Colors arranges in hue/saturation space

c = canvas.canvas()
outline = path.circle(0,0,7.5) # Draw the hue axis around the edge
c.stroke(outline)

item_list = []
for name,h,s in zip(colors,h_list,s_list): # Convert hue/saturation into x/y coordinates
 x = s*sin(h)*7.5
 y = s*cos(h)*7.5
 item_list.append([x,y,name])

item_list.sort(lambda x,y: cmp(y[0],x[0])) # Draw from right-to-left

boxes = []
positions_l = [ [text.halign.boxleft , text.valign.top] , [text.halign.boxleft , text.valign.bottom] ]
positions_r = [ [text.halign.boxright, text.valign.top] , [text.halign.boxright, text.valign.bottom] ]
for x,y,name in item_list:
 if x<0: positions2 = positions_l + positions_r
 else  : positions2 = positions_r + positions_l
 for position in positions2:
  textbox = c.texrunner.text(x, y, name, position)
  collision = False
  for item in boxes:
   if textbox.bbox().intersects(item.bbox()): # If this color collides with one already drawn, try moving label
    collision = True
    continue
  if not collision: break
 if not collision:
  shell = path.circle(x,y,0.2)
  c.fill(shell, [ pyx.color.cmyk.__dict__[name] ]) # Draw a blob of the appropriate color
  boxes.append(textbox)
 else:
  print name

for box in boxes: c.insert(box) # ... and then label it

c.writeEPSfile("pyx_colors.eps")
c.writePDFfile("pyx_colors.pdf")

# --------------
# Figure 2: A list of colors, sorted alphabetically

c = canvas.canvas()

colors2 = colors[:]
colors2.sort()
Clen = 20 # 20 colors in each column
for i in range(len(colors2)):
 name = colors2[i]
 y =  - (i % Clen) * 0.5
 x = int(i / Clen) * 3.5
 shell = path.circle(x,y,0.2)
 c.fill(shell, [ pyx.color.cmyk.__dict__[name] ])
 c.text(x+0.5,y,name, [ text.halign.boxleft, text.valign.middle ] )

c.writeEPSfile("pyx_colors2.eps")
c.writePDFfile("pyx_colors2.pdf")

# ------------
# Figure 3: A list of colors, sorted by hue

c = canvas.canvas()

colors3 = zip(colors,h_list)
colors3.sort(lambda x,y: cmp(y[1],x[1]))
for i in range(len(colors3)):
 name = colors3[i][0]
 y =  - (i % Clen) * 0.5
 x = int(i / Clen) * 3.5
 shell = path.circle(x,y,0.2)
 c.fill(shell, [ pyx.color.cmyk.__dict__[name] ])
 c.text(x+0.5,y,name, [ text.halign.boxleft, text.valign.middle ] )

c.writeEPSfile("pyx_colors3.eps")
c.writePDFfile("pyx_colors3.pdf")




# makeFigureTex.py
#
# The code in this file is part of PyXPlot
# <http://www.pyxplot.org.uk>
#
# Copyright (C) 2006-2012 Dominic Ford <coders@pyxplot.org.uk>
#
# $Id: makeFigureTex.py 1241 2012-06-22 23:03:46Z dcf21 $
#
# PyXPlot is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# You should have received a copy of the GNU General Public License along with
# PyXPlot; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

# ----------------------------------------------------------------------------

# Turn scripts of examples into latex

import os,glob,re

os.system("rm -Rf examples/tex")
os.system("mkdir  examples/tex")

def line_texify(line):
  if line.startswith("#NC "): line=line[4:]
  line = re.sub(r'examples/eps/ex_','',line)
  line = re.sub(r'examples/ex_','',line)
  line = re.sub(r'examples/','',line)
  line = re.sub(r'[\\]', r'gpzywxqqq', line) # LaTeX does not like backslashs
  line = re.sub(r'[_]', r'\\_', line) # LaTeX does not like underscores....
  line = re.sub(r'[&]', r'\\&', line) # LaTeX does not like ampersands....
  line = re.sub(r'[%]', r'\\%', line) # LaTeX does not like percents....
  line = re.sub(r'[$]', r'\\$', line) # LaTeX does not like $s....
  line = re.sub(r'[{]', r'\\{', line) # LaTeX does not like {s....
  line = re.sub(r'[}]', r'\\}', line) # LaTeX does not like }s....
  line = re.sub(r'[#]', r'\\#', line) # LaTeX does not like #s....
  line = re.sub(r'[\^]', r'\\^{}', line) # LaTeX does not like carets....
  line = re.sub(r'[~]', r'$\\sim$', line) # LaTeX does not like tildas....
  line = re.sub(r'[<]', r'$<$', line) # LaTeX does not like < outside of mathmode....
  line = re.sub(r'[>]', r'$>$', line) # LaTeX does not like > outside of mathmode....
  line = re.sub(r'gpzywxqqq', r'$\\backslash$', line) # LaTeX does not like backslashs
  line = re.sub(r' ', r'~', line)
  return line

def makeTeX(fname, counter, linelist):
  fname  = os.path.join("examples","tex",os.path.split(fname)[1][:-4]+"_%d.tex"%counter)
  output = open(fname,"w")
  fns    = max([len(l) for l in linelist]) > 64
  first  = True
  if fns: output.write("{\\footnotesize\n")
  for line in linelist:
    if (not first):
      output.write("\\newline\n")
    if (len(line.strip())==0):
      if fns: output.write("}\\\\{\\footnotesize\n")
      else  : output.write("\\\\\n")
      first = True
      continue
    first = False
    line = line_texify(line)
    for i in range(len(line)):
      if (line[i]!=' '):
        break;
    if (i>0):
      line2 = "\\phantom{"
      for j in range(i): line2 += "x"
      line2 += "}" + line.strip()
      line = line2
    else:
      line = line.strip()
    output.write("\\noindent\\texttt{%s}"%line)
  if fns: output.write("\n}")
  output.close()

files = glob.glob("examples/ex_*.ppl")
files.sort()
for fname in files:
  print "Converting example to latex <%s>..."%os.path.split(fname)[1]
  buffer    = []
  buffering = False
  counter   = 1
  for line in open(fname):
    if (line.strip()=="# BEGIN"):
      buffering = True
      buffer    = []
      continue
    if (line.strip()=="# END"):
      makeTeX(fname, counter, buffer)
      counter   = counter+1
      buffering = False
      continue
    if (buffering):
      buffer.append(line)

